
% main.m
%
% This script allows to perform some numerical tests by giving the results
% obtained when solving a linear system A*x=b by using the CMRH algorithm
%
% You have to choose :
%    - the size of the system, 
%    - the coefficient matrix, 
%    - the exact solution (so that we can compute the error), or only the right hand-side, 
%    - the tolerance 
%


fprintf('Enter the matrix choice :\n')
fprintf('1- Example 1 \n')
fprintf('2- Hankel matrix \n')
fprintf('3- Brown matrix \n')
fprintf('4- Helsing matrix \n')
fprintf('5- Matrix Market matrix or Florida Matrix Collection \n')
matrixchoice=input('Enter your choice : ');
switch matrixchoice
    case 1
        n=input('Enter the size of the matrix : ');
        epsilon=input('Enter  epsilon : ');
        A = example1(n,epsilon);
    case 2
        n=input('Enter the size of the matrix : '); 
        A = hankel(n);
    case 3
        n=input('Enter the size of the matrix : '); 
        epsilon=input('Enter  epsilon : ');
        A = brown(n,epsilon);
    case 4
        n=input('Enter the size of the matrix : '); 
        A = helsing(n);
    case 5
         filename = input('Enter the matrix file name :','s');
        [A,rows,cols,entries,rep,field,symm] = mmread(filename);
        n = rows;        
end
       

fprintf('Enter the solution choice or the right-hand side :\n')
fprintf('1- sol=random vector \n')
fprintf('2- sol=[1,1,...,1]^T \n')
fprintf('3- sol=[1,2,...,n]^T \n')
fprintf('4- rhs=[1,1,...,1]^T \n')
fprintf('5- Matrix Market or Florida Matrix Collection right-hand side\n')
solchoice = input('Enter your choice : ');
if (solchoice<=3)
    if (solchoice==1)
        sol=rand(n,1);
    end
    if (solchoice==2)
        sol=ones(n,1);
    end
    if (solchoice==3)
        sol=[1:n]';
    end
    b=A*sol;
else
    if (solchoice==4)
        b=ones(n,1);
    end
    if (solchoice==5)
    filename2 = input('Enter the matrix file name :','s');
    [b,brows,bcols,bentries,brep,bfield,bsymm] = mmread(filename2);
    end
end

tol = input('Enter the tolerance : ');
iprint = input('Print results : 0- No, 1- Yes : ');
if (iprint==1)
    if (solchoice>3)
        iprint=2;
    end
end



V = A; bc = b;
[x,pseudores,flag] = CMRH(A,b,tol,n,zeros(n,1),iprint);

fid = fopen('pseudores.dat','w');
for i=1:length(pseudores)
    fprintf(fid,'%i \n',pseudores(i));
end
fclose(fid); 

b = x;
if (iprint==1)
    fprintf('\n approx. and exact solutions :\n');
    if (n < 10)
       for i = 1:n
         fprintf(' %6.4e %6.4e\n',b(i),sol(i));
       end
    else
       for i = 1:10
         fprintf(' %6.4e %6.4e\n',b(i),sol(i));
       end
    end
    % we compute the norm of the residu
    bc = -V*b+bc;
    re_cmrh = norm(bc,2);
    % we compute the norm of the error
    sol = -b+sol;
    er_cmrh = norm(sol,2);
    fprintf('residual norm: %6.4e  error norm: %6.4e \n',re_cmrh,er_cmrh);
end
if (iprint==2)
fprintf('\n approx. solution :\n');
if (n < 10)
for i = 1:n
fprintf(' %6.4e\n',b(i));
end
else
for i = 1:10
fprintf(' %6.4e\n',b(i));
end
end
% we compute the norm of the residu
bc = -V*b+bc;
re_cmrh = norm(bc,2);
fprintf('residual norm: %6.4e\n',re_cmrh);
end